package com.example.nekontakte.core.repository;

import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import com.example.nekontakte.core.model.BaseEntity;

public abstract class MapRepository<E extends BaseEntity> implements BaseRepository<E, Integer> {

    private final Map<Integer, E> entities = new TreeMap<>();
    private Integer lastId = 0;

    private boolean checkNull(E entity) {
        if (get(entity.getId()) == null)
            return false;
        return true;
    }

    protected MapRepository() {
    }

    @Override
    public List<E> getAll() {
        return entities.values().stream().toList();
    }

    @Override
    public E get(Integer id) {
        return entities.get(id);
    }

    @Override
    public E create(E entity) {
        lastId++;
        entity.setId(lastId);
        entities.put(lastId, entity);
        return entity;
    }

    @Override
    public E update(E entity) {
        if (checkNull(entity)) {
            entities.put(entity.getId(), entity);
            return entity;
        }
        return null;
    }

    @Override
    public E delete(E entity) {
        if (checkNull(entity)) {
            entities.remove(entity.getId());
            return entity;
        }
        return null;
    }

    @Override
    public void deleteAll() {
        entities.clear();
        lastId = 0;
    }
}
