package com.example.nekontakte.users.api;

import com.example.nekontakte.core.api.PageDtoMapper;
import com.example.nekontakte.core.api.PageDto;

import com.example.nekontakte.core.configurations.Constants;
import com.example.nekontakte.users.model.UserEntity;
import com.example.nekontakte.users.service.UserService;

import jakarta.validation.Valid;

import org.modelmapper.ModelMapper;
import org.springframework.data.domain.PageRequest;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping(Constants.API_URL + "/user")
public class UserController {

    private final UserService userService;
    private final ModelMapper modelMapper;

    public UserController(UserService userService, ModelMapper modelMapper) {
        this.modelMapper = modelMapper;
        this.userService = userService;
    }

    private UserEntity toEntity(UserDTO dto) {
        return modelMapper.map(dto, UserEntity.class);
    }

    private UserDTO toDTO(UserEntity entity) {
        return modelMapper.map(entity, UserDTO.class);
    }

    @GetMapping
    public PageDto<UserDTO> getAll(@RequestParam(name = "pageNumber", defaultValue = "0") Integer pageNumber,
            @RequestParam(name = "pageSize", defaultValue = "5") Integer pageSize) {
        PageRequest pageRequest = PageRequest.of(pageNumber, pageSize);
        return PageDtoMapper.toDto(userService.getAll(pageRequest), this::toDTO);
    }

    @GetMapping("/{id}")
    public UserDTO get(@PathVariable(name = "id") Integer id) {
        return toDTO(userService.get(id));
    }

    @PostMapping
    public UserDTO create(@RequestBody @Valid UserDTO userDTO) {
        return toDTO(userService.create(toEntity(userDTO)));
    }

    @PutMapping("/{id}")
    public UserDTO update(@PathVariable(name = "id") Integer id, @RequestBody UserDTO userDTO) {
        return toDTO(userService.update(id, toEntity(userDTO)));
    }

    @DeleteMapping("/{id}")
    public UserDTO delete(@PathVariable(name = "id") Integer id) {
        return toDTO(userService.delete(id));
    }

}