package com.example.nekontakte.users.model;

import java.util.Date;
import java.util.Objects;
import java.util.HashSet;
import java.util.Set;

import com.example.nekontakte.core.model.BaseEntity;
import com.example.nekontakte.posts.model.PostEntity;

import jakarta.persistence.CascadeType;
import jakarta.persistence.Column;
import jakarta.persistence.OneToMany;
import jakarta.persistence.OrderBy;
import jakarta.persistence.Entity;
import jakarta.persistence.Table;
import jakarta.persistence.Temporal;
import jakarta.persistence.TemporalType;

@Entity
@Table(name = "users")
public class UserEntity extends BaseEntity {
    @Column(nullable = false)
    private String name;
    @Column(nullable = false)
    private String surname;
    @Temporal(value = TemporalType.DATE)
    @Column(nullable = false)
    private Date birthday;
    @Column
    private String city;
    @Column
    private String avatarImg;
    @Column(nullable = false, unique = true)
    private String username;
    @Column(nullable = false)
    private String password;
    @Column(nullable = false)
    private boolean isAdmin;
    @Column
    private String status;
    @OneToMany(mappedBy = "user", cascade = CascadeType.ALL)
    @OrderBy("id ASC")
    private Set<PostEntity> posts = new HashSet<>();

    public UserEntity() {
    }

    public UserEntity(
            String name,
            String surname,
            Date birthday,
            String username,
            String password,
            boolean isAdmin) {
        setName(name);
        setSurname(surname);
        setBirthday(birthday);
        setUsername(username);
        setPassword(password);
        setIsAdmin(isAdmin);
    }

    public Set<PostEntity> getPosts() {
        return posts;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getSurname() {
        return surname;
    }

    public void setSurname(String surname) {
        this.surname = surname;
    }

    public Date getBirthday() {
        return birthday;
    }

    public void setBirthday(Date birthday) {
        this.birthday = birthday;
    }

    public String getCity() {
        return city;
    }

    public void setCity(String city) {
        this.city = city;
    }

    public String getAvatarImg() {
        return avatarImg;
    }

    public void setAvatarImg(String avatarImg) {
        this.avatarImg = avatarImg;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public boolean getIsAdmin() {
        return isAdmin;
    }

    public void setIsAdmin(boolean isAdmin) {
        this.isAdmin = isAdmin;
    }

    @Override
    public int hashCode() {
        return Objects.hash(
                id,
                username,
                password,
                isAdmin,
                name,
                surname,
                status,
                city,
                birthday,
                avatarImg);
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null || getClass() != obj.getClass())
            return false;
        final UserEntity other = (UserEntity) obj;
        return Objects.equals(other.getId(), id) &&
                Objects.equals(other.getUsername(), username) &&
                Objects.equals(other.getIsAdmin(), isAdmin) &&
                Objects.equals(other.getPassword(), password) &&
                Objects.equals(other.getName(), name) &&
                Objects.equals(other.getSurname(), surname) &&
                Objects.equals(other.getStatus(), status) &&
                Objects.equals(other.getCity(), city) &&
                Objects.equals(other.getBirthday(), birthday) &&
                Objects.equals(other.getAvatarImg(), avatarImg);
    }
}