package com.example.nekontakte.users.service;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;

import com.example.nekontakte.core.errors.NotFoundException;
import com.example.nekontakte.users.model.UserEntity;
import com.example.nekontakte.users.repository.UserRepository;

import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.StreamSupport;

@Service
public class UserService {
    private final UserRepository repository;

    private boolean checkUsernameIsExist(String username) {
        return repository.findByUsernameIgnoreCase(username).isPresent();
    }

    public UserService(UserRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    public Page<UserEntity> getAll(PageRequest pageRequest) {
        return repository.findAll(pageRequest);
    }

    @Transactional(readOnly = true)
    public List<UserEntity> getAll() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public UserEntity get(Integer id) {
        return repository.findById(id).orElseThrow(() -> new NotFoundException(UserEntity.class, id));
    }

    @Transactional
    public UserEntity create(UserEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        if (checkUsernameIsExist(entity.getUsername())) {
            throw new IllegalArgumentException(
                    String.format("User with username %s is already exists", entity.getUsername()));
        }
        return repository.save(entity);
    }

    @Transactional
    public UserEntity update(Integer id, UserEntity entity) {
        final UserEntity existsentity = get(id);
        var usersWithSameUsername = repository.findByUsernameIgnoreCase(entity.getUsername());
        if (usersWithSameUsername.get().getId() != existsentity.getId()) {
            throw new IllegalArgumentException(
                    String.format("User with username %s is already exists", entity.getUsername()));
        }
        existsentity.setUsername(entity.getUsername());
        existsentity.setPassword(entity.getPassword());
        existsentity.setIsAdmin(entity.getIsAdmin());
        existsentity.setName(entity.getName());
        existsentity.setSurname(entity.getSurname());
        existsentity.setStatus(entity.getStatus());
        existsentity.setBirthday(entity.getBirthday());
        existsentity.setCity(entity.getCity());
        existsentity.setStatus(entity.getStatus());
        repository.save(existsentity);
        return existsentity;
    }

    @Transactional
    public UserEntity delete(Integer id) {
        final UserEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }
}