package com.example.nekontakte;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Locale;

import org.apache.coyote.BadRequestException;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.nekontakte.core.errors.NotFoundException;
import com.example.nekontakte.posts.model.PostEntity;
import com.example.nekontakte.posts.service.PostService;
import com.example.nekontakte.users.model.UserEntity;
import com.example.nekontakte.users.service.UserService;

import org.junit.jupiter.api.TestMethodOrder;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
public class PostServiceTests {
        @Autowired
        private UserService userService;

        @Autowired
        private PostService postService;

        private UserEntity firstUser;
        private UserEntity secondUser;

        private PostEntity firstPost;
        private PostEntity secondPost;
        private PostEntity thirdPost;

        @BeforeEach
        void createData() throws ParseException, BadRequestException {
                removeData();
                SimpleDateFormat dateFormatter = new SimpleDateFormat("dd.MM.yyyy", Locale.ENGLISH);
                SimpleDateFormat dateTimeFormatter = new SimpleDateFormat("dd.MM.yyyy, HH:mm:ss", Locale.ENGLISH);

                firstUser = new UserEntity();
                firstUser.setUsername("nspotapov");
                firstUser.setPassword("pass123456");
                firstUser.setIsAdmin(true);
                firstUser.setName("Никита");
                firstUser.setSurname("Потапов");
                firstUser.setCity("Ульяновск");
                firstUser.setStatus("Я здесь админ");
                firstUser.setBirthday(dateFormatter.parse("17.02.2003"));

                secondUser = new UserEntity();
                secondUser.setUsername("ekallin");
                secondUser.setPassword("pass87654321");
                secondUser.setIsAdmin(false);
                secondUser.setName("Елена");
                secondUser.setSurname("Каллин");
                secondUser.setCity("Новоульяновск");
                secondUser.setBirthday(dateFormatter.parse("14.06.2005"));

                firstUser = userService.create(firstUser);
                secondUser = userService.create(secondUser);

                firstPost = new PostEntity();
                firstPost.setUser(firstUser);
                firstPost.setPubDate(dateFormatter.parse("01.04.2024"));
                firstPost.setHtml("Первый тестовый пост");

                firstPost = postService.create(firstPost);

                secondPost = new PostEntity();
                secondPost.setUser(firstUser);
                secondPost.setPubDate(dateTimeFormatter.parse("22.03.2024, 09:34:01"));
                secondPost.setImage("myimage.jpg");
                secondPost.setHtml("Второй тестовый пост");

                secondPost = postService.create(secondPost);

                thirdPost = new PostEntity();
                thirdPost.setUser(secondUser);
                thirdPost.setPubDate(dateTimeFormatter.parse("13.02.2024, 18:01:01"));
                thirdPost.setImage("ohmypost.jpg");
                thirdPost.setHtml("Третий постовый пост");

                thirdPost = postService.create(thirdPost);
        }

        @AfterEach
        void removeData() {
                postService.getAll().forEach(item -> postService.delete(item.getId()));
                userService.getAll().forEach(item -> userService.delete(item.getId()));
        }

        @Test
        void getTest() {
                Assertions.assertThrows(NotFoundException.class, () -> postService.get(0));
        }

        @Test
        @Order(1)
        void createTest() throws ParseException, BadRequestException {
                SimpleDateFormat dateTimeFormatter = new SimpleDateFormat("dd.MM.yyyy, HH:mm:ss", Locale.ENGLISH);

                PostEntity fourthPost = new PostEntity();
                fourthPost.setUser(secondUser);
                fourthPost.setPubDate(dateTimeFormatter.parse("13.02.2024, 18:01:01"));

                Assertions.assertThrows(BadRequestException.class,
                                () -> postService.create(fourthPost));

                PostEntity test = postService.get(firstPost.getId());

                Assertions.assertEquals(firstPost, test);
                Assertions.assertEquals(firstUser, postService.get(firstPost.getId()).getUser());
                Assertions.assertEquals(secondUser, postService.get(thirdPost.getId()).getUser());
                Assertions.assertEquals(3, postService.getAll().size());
                Assertions.assertEquals(2, postService.getAllByUserId(firstUser.getId()).size());
        }

        @Test
        @Order(2)
        void update() throws BadRequestException, ParseException {
                Integer postId = firstPost.getId();
                PostEntity oldPost = postService.get(postId);
                String oldImage = oldPost.getImage();

                PostEntity newPost = new PostEntity();
                newPost.setId(oldPost.getId());
                newPost.setUser(oldPost.getUser());
                newPost.setPubDate(oldPost.getPubDate());
                newPost.setImage("new image");
                newPost.setHtml(oldPost.getHtml());

                postService.update(postId, newPost);
                Assertions.assertNotEquals(postService.get(postId).getImage(), oldImage);
        }

        @Test
        @Order(3)
        void delete() {
                final int oldCount = postService.getAll().size();
                postService.delete(firstPost.getId());
                Assertions.assertEquals(oldCount - 1, postService.getAll().size());
        }
}
