package com.example.nekontakte;

import org.junit.jupiter.api.TestMethodOrder;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Locale;

import org.apache.coyote.BadRequestException;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.nekontakte.core.errors.NotFoundException;
import com.example.nekontakte.subscribes.model.SubscribeEntity;
import com.example.nekontakte.subscribes.service.SubscribeService;
import com.example.nekontakte.users.service.UserService;
import com.example.nekontakte.users.model.UserEntity;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
public class SubscribeServiceTests {
        @Autowired
        private UserService userService;

        @Autowired
        private SubscribeService subscribeService;
        private UserEntity firstUser;
        private UserEntity secondUser;
        private UserEntity thirdUser;

        private SubscribeEntity secondSubscribe;

        @BeforeEach
        void createData() throws ParseException, BadRequestException {
                removeData();
                SimpleDateFormat dateFormatter = new SimpleDateFormat("dd.MM.yyyy", Locale.ENGLISH);

                UserEntity firstUserEntity = new UserEntity();
                firstUserEntity.setUsername("nspotapov");
                firstUserEntity.setPassword("pass123456");
                firstUserEntity.setIsAdmin(true);
                firstUserEntity.setName("Никита");
                firstUserEntity.setSurname("Потапов");
                firstUserEntity.setCity("Ульяновск");
                firstUserEntity.setStatus("Я здесь админ");
                firstUserEntity.setBirthday(dateFormatter.parse("17.02.2003"));

                UserEntity secondUserEntity = new UserEntity();
                secondUserEntity.setUsername("ekallin");
                secondUserEntity.setPassword("pass87654321");
                secondUserEntity.setIsAdmin(false);
                secondUserEntity.setName("Елена");
                secondUserEntity.setSurname("Каллин");
                secondUserEntity.setCity("Новоульяновск");
                secondUserEntity.setBirthday(dateFormatter.parse("14.06.2005"));

                UserEntity thirdUserEntity = new UserEntity();
                thirdUserEntity.setUsername("olegulya");
                thirdUserEntity.setPassword("passOleg");
                thirdUserEntity.setIsAdmin(false);
                thirdUserEntity.setName("Олег");
                thirdUserEntity.setSurname("Тинькофф");
                thirdUserEntity.setCity("Полысаево");
                thirdUserEntity.setStatus(
                                "Вчера я потерял $250 млн за день, были дни и похуже, миллиарды в день терял. Поэтому это позитивный очень день");
                thirdUserEntity.setBirthday(dateFormatter.parse("25.12.1967"));

                firstUser = userService.create(firstUserEntity);
                secondUser = userService.create(secondUserEntity);
                thirdUser = userService.create(thirdUserEntity);

                SubscribeEntity subscribeEntity = new SubscribeEntity();
                subscribeEntity.setUser(firstUser);
                subscribeEntity.setSubscriber(secondUser);

                subscribeService.create(subscribeEntity);

                subscribeEntity = new SubscribeEntity();
                subscribeEntity.setUser(firstUser);
                subscribeEntity.setSubscriber(thirdUser);

                secondSubscribe = subscribeService.create(subscribeEntity);

                subscribeEntity = new SubscribeEntity();
                subscribeEntity.setUser(secondUser);
                subscribeEntity.setSubscriber(thirdUser);

                subscribeService.create(subscribeEntity);
        }

        @AfterEach
        void removeData() {
                subscribeService.getAll().forEach(item -> subscribeService.delete(item.getId()));
                userService.getAll().forEach(item -> userService.delete(item.getId()));
        }

        @Test
        void getTest() {
                Assertions.assertThrows(NotFoundException.class, () -> subscribeService.get(0));
        }

        @Test
        @Order(1)
        void createTest() throws ParseException, BadRequestException {
                Assertions.assertEquals(secondSubscribe, subscribeService.get(secondSubscribe.getId()));
                Assertions.assertEquals(3, subscribeService.getAll().size());
                Assertions.assertEquals(2, subscribeService.getAllByUserId(firstUser.getId()).size());
        }

        @Test
        @Order(2)
        void update() throws BadRequestException, ParseException {
                Integer subscribeId = secondSubscribe.getId();
                SubscribeEntity oldSubscribeEntity = subscribeService.get(subscribeId);
                UserEntity oldSubscriber = oldSubscribeEntity.getSubscriber();

                SubscribeEntity newSubscribeEntity = new SubscribeEntity();
                newSubscribeEntity.setId(oldSubscribeEntity.getId());
                newSubscribeEntity.setUser(oldSubscribeEntity.getUser());
                newSubscribeEntity.setSubscriber(secondUser);

                subscribeService.update(subscribeId, newSubscribeEntity);
                Assertions.assertNotEquals(subscribeService.get(subscribeId).getSubscriber(), oldSubscriber);
        }

        @Test
        @Order(3)
        void delete() {
                Integer oldCount = subscribeService.getAll().size();
                subscribeService.delete(secondSubscribe.getId());
                Assertions.assertEquals(oldCount - 1, subscribeService.getAll().size());
        }
}
