package com.example.nekontakte;

import org.junit.jupiter.api.TestMethodOrder;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Locale;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.nekontakte.core.errors.NotFoundException;
import com.example.nekontakte.users.service.UserService;
import com.example.nekontakte.users.model.UserEntity;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
public class UserServiceTests {

	@Autowired
	private UserService userService;

	private UserEntity firstUser;

	@BeforeEach
	void createData() throws ParseException {
		removeData();
		SimpleDateFormat dateFormatter = new SimpleDateFormat("dd.MM.yyyy", Locale.ENGLISH);

		UserEntity firstUserEntity = new UserEntity();
		firstUserEntity.setUsername("nspotapov");
		firstUserEntity.setPassword("pass123456");
		firstUserEntity.setIsAdmin(true);
		firstUserEntity.setName("Никита");
		firstUserEntity.setSurname("Потапов");
		firstUserEntity.setCity("Ульяновск");
		firstUserEntity.setStatus("Я здесь админ");
		firstUserEntity.setBirthday(dateFormatter.parse("17.02.2003"));

		UserEntity secondUserEntity = new UserEntity();
		secondUserEntity.setUsername("ekallin");
		secondUserEntity.setPassword("pass87654321");
		secondUserEntity.setIsAdmin(false);
		secondUserEntity.setName("Елена");
		secondUserEntity.setSurname("Каллин");
		secondUserEntity.setCity("Новоульяновск");
		secondUserEntity.setBirthday(dateFormatter.parse("14.06.2005"));

		UserEntity thirdUserEntity = new UserEntity();
		thirdUserEntity.setUsername("olegulya");
		thirdUserEntity.setPassword("passOleg");
		thirdUserEntity.setIsAdmin(false);
		thirdUserEntity.setName("Олег");
		thirdUserEntity.setSurname("Тинькофф");
		thirdUserEntity.setCity("Полысаево");
		thirdUserEntity.setStatus(
				"Вчера я потерял $250 млн за день, были дни и похуже, миллиарды в день терял. Поэтому это позитивный очень день");
		thirdUserEntity.setBirthday(dateFormatter.parse("25.12.1967"));

		firstUser = userService.create(firstUserEntity);
		userService.create(secondUserEntity);
		userService.create(thirdUserEntity);
	}

	@AfterEach
	void removeData() {
		userService.getAll().forEach(item -> userService.delete(item.getId()));
	}

	@Test
	void getTest() {
		Assertions.assertThrows(NotFoundException.class, () -> userService.get(0));
	}

	@Test
	@Order(1)
	void createTest() throws ParseException {
		Assertions.assertEquals(3, userService.getAll().size());
	}

	@Test
	@Order(2)
	void update() {
		final String newPassword = "newpassword";
		final UserEntity existEntity = userService.get(firstUser.getId());
		final String oldPassword = existEntity.getPassword();

		final UserEntity entity = new UserEntity();
		entity.setUsername(existEntity.getUsername());
		entity.setPassword(newPassword);
		entity.setIsAdmin(existEntity.getIsAdmin());
		entity.setName(existEntity.getName());
		entity.setSurname(existEntity.getSurname());
		entity.setBirthday(existEntity.getBirthday());
		entity.setCity(existEntity.getCity());
		entity.setAvatarImg(existEntity.getAvatarImg());
		entity.setStatus(existEntity.getStatus());

		final UserEntity newEntity = userService.update(firstUser.getId(), entity);
		Assertions.assertEquals(newPassword, newEntity.getPassword());
		Assertions.assertNotEquals(oldPassword, newEntity.getPassword());
	}

	@Test
	@Order(3)
	void delete() {
		userService.delete(firstUser.getId());
		Assertions.assertEquals(2, userService.getAll().size());
	}
}
